import argparse
from pyspark import SparkContext
from pyspark import SparkConf
import subprocess
import os
import traceback
#start docker containter
# docker run -v /data/MEP_Projects/Frame:/data/projects/Frame:ro -v ~/Private/Frame/output:/data/output:rw -it frame_smooth
#frame_s2smooth --output-end 2021-03-10 --input-end 2021-03-10 --input-dekads 36 --output-dekads 6 --output-template s2t_{prd}_{dekad} --mask /data/projects/Frame/nrt/S2/REF/S2_SRI_mask.tif /data/projects/Frame//nrt/S2/TILES/44PMQ/TIF/ /data/output/TILES/44PMQ/T10/2107/ --dump-metadata /data/output/logs/44PMQ_2107.log
#

def parse_arguments():

    parser = argparse.ArgumentParser(prog='smooth_all_tiles.py')

    parser.add_argument("--output-end", type=str, help="output end-date",
                        required=True)
    parser.add_argument("--input-end", type=str, help="input end-date",
                        required=True)
    parser.add_argument("--input-dekads", type=int,
                        help="number of inputs dekads to use",
                        required=False,
                        default=36)
    parser.add_argument("--input-start", type=str, help="input start-date",
                        required=False,
                        default=None)
    parser.add_argument("--output-dekads", type=int,
                        help="number of output dekads to use",
                        default=6,
                        required=False)
    parser.add_argument("--output-template", type=str, help="output-file-format",
                        required=False,
                        default='s2t_{prd}_{dekad}')
    parser.add_argument("--mask", type=str, help="the mask",
                        required=False,
                        default='/data/projects/Frame/nrt/S2/REF/S2_AFR_mask.tif')
    parser.add_argument("--input-dir", type=str, help="input folder",
                        required=True)
    parser.add_argument("--dekad", type=str, help="dekad to process",
                        required=True)
    parser.add_argument("--output-dir", type=str,
                        help="output folder",
                        required=True)
    parser.add_argument("--tile-list", type=str,
                        help="file with tiles",
                        required=True)
    return parser.parse_args()




def run_command(cmd):
    p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    stdout, stderr = p.communicate()
    err_nr = p.returncode
    return str(stdout), str(stderr), err_nr

def smooth( output_end, input_end, input_dekads, output_dekads, mask, input_dir, output_dir, logdir, tile, dekad, input_start_date=None):
    tile = tile.strip()
    # we use input start date for historical processing and input-dekads for nrt processing
    print ("input_start_date = %s" % input_start_date)

    if input_start_date != None:
        input_par = "--input-start %s" % input_start_date
    else:
        input_par = "--input-dekads %s" % input_dekads

    #input_par ="--input-start %s" % input_start_date if input_start_date != None else "--input-dekads %s" % input_dekads

    cmd = "frame_s2smooth --output-end %s --input-end %s %s --output-dekads %s --output-template s2t_{prd}_{dekad} --mask %s %s/%s/ %s/%s/%s --dump-metadata %s/%s/%s.log" % (
    #cmd = "frame_s2smooth --output-end %s --input-end %s --input-dekads %s --output-dekads %s --output-template s2t_{prd}_{dekad} --mask %s %s/%s/ %s/%s/%s --dump-metadata %s/%s/%s.log" % (
    #cmd = "frame_s2smooth --output-end %s --input-end %s --input-dekads %s --output-dekads %s --output-template s2t_{prd}_{dekad} --mask %s %s/%s/TIF/ %s/%s/T10/ --dump-metadata /dev/stdout"  % (
        output_end,
        input_end,
        input_par,
        output_dekads,
        mask,
        input_dir,
        tile,
        output_dir,
        tile,
        dekad,
        logdir,
        dekad,
        tile)
    print(cmd)

    stdout, stderr, err_nr = run_command(cmd)
    if err_nr:
        print (traceback.print_exc())
        print (stdout)
        print (stderr)
        raise Exception("smoothing failed")
    return 0

def main():
    args = parse_arguments()
    print(args)

    sc_conf = SparkConf()
    sc = SparkContext(conf=sc_conf)


    try:
        with open(args.tile_list) as f:
            tiles = f.readlines()
        tiles = list(map(lambda tile: tile.replace('"',''), tiles))  # remove " around tiles
        number_of_tiles = len(tiles)
        if number_of_tiles > 0:
            logdir = os.path.join(args.output_dir, 'logs')
            tiles_rdd = sc.parallelize(tiles, len(tiles))
            tiles_rdd.foreach(lambda tile: smooth(args.output_end, args.input_end,args.input_dekads, args.output_dekads, args.mask, args.input_dir, args.output_dir, logdir, tile, args.dekad, input_start_date=args.input_start))

    finally:
        sc.stop()


if __name__ == "__main__":
    main()

