import argparse
from pyspark import SparkContext
from pyspark import SparkConf
import os
from os.path import join
import json

from frame_s2.mosaicer import mosaic_and_clip

def parse_arguments():

    parser = argparse.ArgumentParser(prog='wrapper_mosaicer.py')

    parser.add_argument("--start_dekad", type=str,
                        help="first dekad to process",
                        required=True)
    parser.add_argument("--end_dekad", type=str, help="last dekad to process",
                        required=True)
    parser.add_argument("--mask", type=str, help="the mask",
                        required=True)
    parser.add_argument("--input_dir", type=str, help="input folder without tileid",
                        required=True)
    parser.add_argument("--output_dir", type=str,
                        help="output folder",
                        required=True)
    parser.add_argument("--tiledir_posix", type=str, help="input folder",
                        required=False, default=None)
    parser.add_argument("--tile_list", type=str,
                        help="file with tiles",
                        required=True)
    parser.add_argument("--roi", type=str,
                        help="roi",
                        required=True)
    parser.add_argument("--clip", type=bool,
                        help="clip the image to the <roi> defined in the config file",
                        default=True)
    return parser.parse_args()

##############################################################################
def previousDekad(dekad):
      year = int("20" + dekad[0:2])
      dekadnr = int(dekad[2:4])
      if dekadnr==1:
          year= year-1
          dekadnr=36
      else:
          dekadnr-=1
      previousdekad=str(year)[2:4] + "{:02d}".format(dekadnr)
      return previousdekad

def create_dekad_lst(start_dekad, end_dekad):
    dekad_lst=[end_dekad]
    dekad=end_dekad
    while dekad > start_dekad:
        dekad = previousDekad(dekad)
        dekad_lst.append(dekad)
    return dekad_lst

def main():
    args = parse_arguments()
    print(args)

    sc_conf = SparkConf()
    sc = SparkContext(conf=sc_conf)

    try:
        dekad_lst  = create_dekad_lst(args.start_dekad, args.end_dekad)
        number_of_jobs=len(dekad_lst)

        if number_of_jobs > 0:
            mos_rdd = sc.parallelize(dekad_lst, len(dekad_lst))
            #tile_list, dekad, output_dir, input_dir, mask, tiledir_posix, roi, clip=True
            mos_rdd.foreach(lambda dekad: mosaic_and_clip(args.tile_list, dekad, args.output_dir, args.input_dir, args.mask, args.tiledir_posix, args.roi, clip=args.clip))

    finally:
        sc.stop()


if __name__ == "__main__":
    main()

