import argparse
from pyspark import SparkContext
from pyspark import SparkConf
import os
from os.path import join
import json

from frame_s2.pact import start_pact


def parse_arguments():
    parser = argparse.ArgumentParser(prog='wrapper_pact.py',
                                     description="create NPP ")
    parser.add_argument("--start_dekad", type=str, help="first dekad to process",
                        required=True)
    parser.add_argument("--end_dekad", type=str, help="last dekad to process",
                        required=True)
    parser.add_argument("--in_fapar_dir", type=str, help="FAPAR image",
                        required=True)
    parser.add_argument("--in_pmax_dir", type=str, help="pmax image",
                        required=True)
    parser.add_argument("--in_classification", type=str, help="classification image",
                        required=True)
    parser.add_argument("--in_RUE", type=str, help="LU-class, RUE, text file",
                        required=True)
    parser.add_argument("--out_npp_dir", type=str, help="output npp file",
                        required=True)
    parser.add_argument("--out_nodata", type=int, help="nodata value for output file",
                        required=False,
                        default=-9999)
    parser.add_argument("--roi", type=str, help="the roi",
                        required=True)
    parser.add_argument("--level", type=int, help="1:Lv1, 2:Lv2",
                        required=False,
                        default=2)
    return parser.parse_args()

##############################################################################
def previousDekad(dekad):
      year = int("20" + dekad[0:2])
      dekadnr = int(dekad[2:4])
      if dekadnr==1:
          year= year-1
          dekadnr=36
      else:
          dekadnr-=1
      previousdekad=str(year)[2:4] + "{:02d}".format(dekadnr)
      return previousdekad

def create_dekad_lst(start_dekad, end_dekad):
    dekad_lst=[end_dekad]
    dekad=end_dekad
    while dekad > start_dekad:
        dekad = previousDekad(dekad)
        dekad_lst.append(dekad)
    return dekad_lst

def main():
    args = parse_arguments()
    print(args)

    sc_conf = SparkConf()
    sc = SparkContext(conf=sc_conf)

    try:
        dekad_lst  = create_dekad_lst(args.start_dekad, args.end_dekad)
        number_of_jobs = len(dekad_lst)

        # validate lst to detect & prevent doubles
        detection_lst = []
        for s10 in dekad_lst:
            detection_lst.append(s10)
        detection_set = set(detection_lst)
        if len(detection_set) < len(detection_lst):
            print("-W- Some double dekad items are filtered from this list!!!")
            dekad_lst = list(set(dekad_lst))  # fitler double items by putting them in a set, convert to a list afterwards

        if number_of_jobs > 0:
            mos_rdd = sc.parallelize(dekad_lst, len(dekad_lst))
            #tile_list, dekad, output_dir, input_dir, mask, tiledir_posix, roi, clip=True
            mos_rdd.foreach(lambda dekad: start_pact(args.in_fapar_dir, args.in_pmax_dir, args.in_classification,
                                                     args.in_RUE, args.out_npp_dir,
                                                     dekad, args.roi, out_nodata=args.out_nodata, level=args.level))

    finally:
        sc.stop()


if __name__ == "__main__":
    main()

