import argparse
from pyspark import SparkContext
from pyspark import SparkConf
from os.path import join
import os
import glob
import re

from frame_s2.postprocess import start_postproc
from frame_s2.postproc_helper import previousDekad

def parse_arguments():
    parser = argparse.ArgumentParser(prog='process_all_tiles.py')

    parser.add_argument("--roi", type=str, help="roi",
                        required=True)
    parser.add_argument("--mosDir", type=str, help="directory of the mosaics, this can be more than one. All files within this directory will be handled",
                        required=False)
    parser.add_argument("--outputDir", type=str, help="Output root directory",
                        required=True)
    return parser.parse_args()

def main():
    args = parse_arguments()
    print(args)

    sc_conf = SparkConf()
    sc = SparkContext(conf=sc_conf)

    #create folder structure if not exists
    dirs2check=[]
    dirs2check.append(join(args.outputDir, 'alb'))
    dirs2check.append(join(args.outputDir, 'ndv'))
    dirs2check.append(join(args.outputDir, 'fpr'))
    dirs2check.append(join(args.outputDir, 'alb/smo'))
    dirs2check.append(join(args.outputDir, 'ndv/smo'))
    dirs2check.append(join(args.outputDir, 'fpr/smo'))
    try:
        for postdir in dirs2check:
            if not os.path.exists(postdir): os.mkdir(postdir)
    except:
        raise Exception("Unable to create postprocessing dir %s " % postdir)
    try:
        #list all dekads in folder, this can be an archive folder where dekads over multiple years are located,
        # or it is an nrt dir, where only 6 dekads are located

        pattern = join(args.mosDir, "composite_NDV_*.tif")
        NDV_lst = glob.glob(pattern)


        dekad_lst = []
        for NDV in NDV_lst:

            m = re.search( "composite_NDV_([0-9]{4}).tif", os.path.basename(NDV))
            if m:
                dekad = m.group(1)
            else:
                raise Exception("Unable to get dekad from filestring %s" % os.path.basename(NDV))
            dekad_lst.append(dekad)

        #sometimes doubles are in the file, use a set to remove the doubles
        date_set = set(dekad_lst)
        dekad_lst = list(date_set)
        dekad_lst.sort()
        number_of_dates = len(dekad_lst)

        #get the dekad number before first dekad from list. We are sure that this dekad doesn't change during parallel processing
        first_dekad_under_processing = dekad_lst[0]
        hist_dekad = previousDekad(first_dekad_under_processing)
        qua_dir = join(args.outputDir, 'ndv', 'qua')
        file_prefix = 'L2' if args.roi == 'AFR' else 'L2_%s' % args.roi
        hist_qua_file = join(qua_dir, '%s_QUAL_%s.tif') % (file_prefix, hist_dekad)

        if number_of_dates > 0:
            pmax_rdd = sc.parallelize(dekad_lst, len(dekad_lst))
            pmax_rdd.foreach(lambda dekad: start_postproc(args.mosDir, args.outputDir, args.roi, dekad, prev_qua_dek_img=hist_qua_file))
        else:
            raise Exception("There is nothing to do. Make sure %s is filled with mos files!" % args.mosDir)

    finally:
        sc.stop()


if __name__ == "__main__":
    main()

